# Vue.js Helper Libs
![npm](https://img.shields.io/npm/v/@mornya/vue-helper)
![node](https://img.shields.io/node/v/@mornya/vue-helper)
![NPM](https://img.shields.io/npm/l/@mornya/vue-helper)
<br>Vue.js Helpers for maintain of the application on bootstrap.

> This project was generated by [Vessel](https://www.npmjs.com/package/@mornya/vessel). For a simple and quick reference, click [here](VESSEL.md).

## About
Vue.js 프로젝트 부트스트래핑을 위한 라이브러리.

## Installation
해당 모듈을 사용할 프로젝트에서는 아래와 같이 설치한다.
```bash
$ npm install --save @mornya/vue-helper
or
$ yarn add @mornya/vue-helper
```

## Usage
src/main.js에서 각 router, store, plugins 등의 설정을 옵션으로 넘겨준다.
```vue
import Vue from 'vue'
import Vuex from 'vuex'
import * as appPlugins from '@/plugins'
import * as appRouter from '@/router'
import * as appModulesStore from '@/store'
import appRootStore from '@/store/root'
import App from '@/App'

const { buildEnv, router, store } = Maintainer.initialize(Vue, {
  showLogEnvs: ['local', 'dev'],
  router: {
    routes: appRouter,
    fallback: {
      // fallback to 404
      path: '/*',
      name: 'application.404',
      component: () => import(/* webpackChunkName: "application.404" */ '@/views/Application/PageNotFound.vue'),
    },
    option: {
      // Vue-Router 옵션
      base: process.env.BASE_URL,
      scrollBehavior () {
        return { x: 0, y: 0 }
      },
    },
    navigationGuards: {
      // NavigationGuards용 함수 선언
      beforeEach (to, from) {},
      beforeResolve (to, from) {},
      afterEach (to, from) {},
    },
  },
  store: {
    vuex: Vuex,
    root: appRootStore,
    modules: appModulesStore,
  },
  plugins: appPlugins,
  serviceWorker: {
    isRegister: process.env.NODE_ENV === 'production',
    base: '/',
    file: 'service-worker.js',
  },
  filter: {},
  prototype: {}, // 정상 처리되지만 IDE에서 코드인식에 문제 있음 (Vue.prototype 사용권장).
  component: {}, // 정상 처리되지만 IDE에서 코드인식에 문제 있음 (Vue.component 사용권장).
  mixin: {},
  directive: {},
})

// 해당 라이브러리 내에서 설정하므로 별도로 아래처럼 선언할 필요는 없다.
// Vue.prototype.$env = buildEnv

// Vue 실행
new Vue({
  router,
  store,
  render: h => h(App),
}).$mount('#app')
```

## Change Log
해당 프로젝트의 [CHANGELOG.md](CHANGELOG.md) 파일 참조.

## License
해당 프로젝트의 [LICENSE](LICENSE) 파일 참조.
